"""Module providing base URLs and endpoints dataclass.
"""

from dataclasses import dataclass

BaseURL = "https://rmlab.net"

ResourceUserLogin = "/auth/user/login"
ResourceGetEndpoints = "/api/endpoints"


@dataclass
class ApiEndpoints:
    """Dataclass to store API endpoints. Initialized after after login."""

    summary: str = str(None)
    data_bounded: str = str(None)
    data_unbounded: str = str(None)
    data_unbounded_ids: str = str(None)
    data_unbounded_fields: str = str(None)
    data_flight: str = str(None)
    data_flight_post: str = str(None)
    data_airline_locations: str = str(None)
    data_reset: str = str(None)
    operation_trigger: str = str(None)
    operation_checkpoint: str = str(None)


@dataclass
class ApiEndpointsLimits:
    """Dataclass to store request limits of some API endpoints. Initialized after login."""

    data_unbounded_fields: int = 100
    data_flight: int = 100
    data_flight_post: int = 100



import os, pathlib
from rmlab.api.fetch.core import APIFetchCore
from rmlab.api.operations.simulation import APISimulation
from rmlab.api.upload.core import APIUploadCore
from rmlab.api.upload.parametric.filters import APIUploadParametric
from rmlab.api.upload.parametric.customers import APIUploadCustomersModels
from rmlab.api.upload.parametric.pricing import APIUploadPricingModels

from rmlab._data.enums import ScenarioDayStatus, ScenarioState

_DataSamplesPath = (
    str(pathlib.Path(os.path.realpath(__file__)).parent.parent) + "/data_samples/"
)


class APISimulationUpload(
    APISimulation,
    APIFetchCore,
    APIUploadCore,
    APIUploadParametric,
    APIUploadCustomersModels,
    APIUploadPricingModels,
):
    pass


async def test_trigger():

    async with APISimulationUpload() as api:

        scen_id = api.scenarios[0]

        # Upload customers models to server
        await api.upload_batch_customers_models(
            scen_id,
            request_models_fns=[
                _DataSamplesPath + "/basic/customers_request.poisson_bl1.json"
            ],
            choice_models_fns=[
                _DataSamplesPath + "/basic/customers_choice.mnl_bl1.json"
            ],
        )

        # Upload pricing models to server
        await api.upload_batch_pricing_models(
            scen_id,
            range_models_fns=[_DataSamplesPath + "/basic/pricing_range.sample.json"],
            behavior_models_fns=[
                _DataSamplesPath + "/basic/pricing_behavior.sample.json"
            ],
            optimizer_models_fns=[
                _DataSamplesPath + "/basic/pricing_optimizer.sample.json"
            ],
        )

        await api.upload_parametric_filters(
            scen_id, _DataSamplesPath + "/basic/table.pfilter.json"
        )

        # Upload core items to server
        await api.upload_batch_core(
            scen_id,
            aircraft_items_fn=_DataSamplesPath + "/basic/table.aircraft.csv",
            airline_items_fn=_DataSamplesPath + "/basic/table.airline.csv",
            airport_items_fn=_DataSamplesPath + "/basic/table.airport.csv",
            city_items_fn=_DataSamplesPath + "/basic/table.city.csv",
            country_items_fn=_DataSamplesPath + "/basic/table.country.csv",
            schedule_items_fn=_DataSamplesPath + "/basic/table.schedule.csv",
        )

        (
            prev_dates,
            prev_items_count,
            prev_schedules_count,
            prev_flights_count,
        ) = await api.fetch_info(scen_id)

        assert all(
            [
                count > 0
                for key, count in prev_items_count.__dict__.items()
                if key != "id"
            ]
        )

        assert prev_dates.current == prev_dates.first_flight_load
        assert prev_dates.next == prev_dates.last_flight_departure
        assert prev_dates.state == ScenarioState.IN_PROGRESS
        assert prev_dates.day_status == ScenarioDayStatus.READY

        assert prev_schedules_count.past == 0
        assert prev_schedules_count.live > 0
        assert prev_schedules_count.pending > 0
        assert prev_schedules_count.total > 0

        assert prev_flights_count.past == 0
        assert prev_flights_count.live > 0
        assert prev_flights_count.pending > 0
        assert prev_flights_count.total > 0

        (
            dates,
            items_count,
            schedules_count,
            flights_count,
        ) = await api.trigger_simulation(scen_id)

        assert dates.current == dates.last_flight_departure
        assert dates.state == ScenarioState.FINISHED
        assert dates.day_status == ScenarioDayStatus.ENDED

        assert all(
            [
                count > 0
                for key, count in prev_items_count.__dict__.items()
                if key != "id"
            ]
        )

        # NOTE: Live flights/schedules can be greater than 0 because non-zero number of flights departed on current
        assert flights_count.live >= 0
        assert flights_count.pending == 0
        assert flights_count.total >= flights_count.past

        assert schedules_count.live >= 0
        assert schedules_count.pending == 0
        assert schedules_count.total >= schedules_count.past



import os, pathlib
from rmlab.api.fetch.core import APIFetchCore
from rmlab.api.fetch.parametric import APIFetchParametric
from rmlab.api.remove import APIRemove
from rmlab.api.upload.core import APIUploadCore
from rmlab.api.upload.parametric.filters import APIUploadParametric
from rmlab.api.upload.parametric.customers import APIUploadCustomersModels
from rmlab.api.upload.parametric.pricing import APIUploadPricingModels

from rmlab._version import __version__

_DataSamplesPath = (
    str(pathlib.Path(os.path.realpath(__file__)).parent.parent) + "/data_samples/"
)


class APIUploadRemove(
    APIUploadCore,
    APIUploadParametric,
    APIUploadCustomersModels,
    APIUploadPricingModels,
    APIFetchParametric,
    APIRemove,
):
    pass


async def test_upload_tables():

    async with APIUploadRemove() as api:

        scen_id = api.scenarios[0]

        await api.remove_data_full(scen_id=scen_id)

        await api.upload_customers_request_model(
            scen_id=scen_id,
            data_fn=_DataSamplesPath + "/basic/customers_request.poisson_bl1.json",
        )

        await api.upload_customers_choice_model(
            scen_id=scen_id,
            data_fn=_DataSamplesPath + "/basic/customers_choice.mnl_bl1.json",
        )

        await api.upload_pricing_range_model(
            scen_id=scen_id,
            data_fn=_DataSamplesPath + "/basic/pricing_range.sample.json",
        )

        await api.upload_pricing_behavior_model(
            scen_id=scen_id,
            data_fn=_DataSamplesPath + "/basic/pricing_behavior.sample.json",
        )

        await api.upload_pricing_optimizer_model(
            scen_id=scen_id,
            data_fn=_DataSamplesPath + "/basic/pricing_optimizer.sample.json",
        )

        await api.upload_parametric_filters(
            scen_id=scen_id, data_fn=_DataSamplesPath + "/basic/table.pfilter.json"
        )

        await api.upload_aircrafts(
            scen_id=scen_id, data_fn=_DataSamplesPath + "/basic/table.aircraft.csv"
        )

        await api.upload_airlines(
            scen_id=scen_id, data_fn=_DataSamplesPath + "/basic/table.airline.csv"
        )

        await api.upload_airports(
            scen_id=scen_id, data_fn=_DataSamplesPath + "/basic/table.airport.csv"
        )

        await api.upload_cities(
            scen_id=scen_id, data_fn=_DataSamplesPath + "/basic/table.city.csv"
        )

        await api.upload_countries(
            scen_id=scen_id, data_fn=_DataSamplesPath + "/basic/table.country.csv"
        )

        await api.upload_schedules(
            scen_id=scen_id, data_fn=_DataSamplesPath + "/basic/table.schedule.csv"
        )


class APIUploadFetchRemove(
    APIUploadCore,
    APIUploadParametric,
    APIUploadCustomersModels,
    APIUploadPricingModels,
    APIFetchCore,
    APIFetchParametric,
    APIRemove,
):
    pass


async def test_reset_upload_get():

    async with APIUploadFetchRemove() as api:

        scen_id = api.scenarios[0]

        await api.remove_data_full(scen_id=scen_id)

        # Upload customers models to server
        await api.upload_batch_customers_models(
            scen_id,
            request_models_fns=[
                _DataSamplesPath + "/basic/customers_request.poisson_bl1.json"
            ],
            choice_models_fns=[
                _DataSamplesPath + "/basic/customers_choice.mnl_bl1.json"
            ],
        )

        # Upload pricing models to server
        await api.upload_batch_pricing_models(
            scen_id,
            range_models_fns=[_DataSamplesPath + "/basic/pricing_range.sample.json"],
            behavior_models_fns=[
                _DataSamplesPath + "/basic/pricing_behavior.sample.json"
            ],
            optimizer_models_fns=[
                _DataSamplesPath + "./basic/pricing_optimizer.sample.json"
            ],
        )

        await api.upload_parametric_filters(
            scen_id, data_fn=_DataSamplesPath + "./basic/table.pfilter.json"
        )

        # Upload core items to server
        await api.upload_batch_core(
            scen_id,
            aircraft_items_fn=_DataSamplesPath + "./basic/table.aircraft.csv",
            airline_items_fn=_DataSamplesPath + "./basic/table.airline.csv",
            airport_items_fn=_DataSamplesPath + "./basic/table.airport.csv",
            city_items_fn=_DataSamplesPath + "./basic/table.city.csv",
            country_items_fn=_DataSamplesPath + "./basic/table.country.csv",
            schedule_items_fn=_DataSamplesPath + "./basic/table.schedule.csv",
        )

        # ---- Assert retrieval of scenario info
        dates, bounded_count, schedules_count, flights_count = await api.fetch_info(
            scen_id
        )

        # ---- Assert all core bounded items are initialized
        arl_items = await api.fetch_airlines(scen_id)
        assert len(arl_items) > 0

        for arl in arl_items:
            arl_sectors_ids = await api.fetch_airline_sectors(scen_id, arl.id)
            assert len(arl_sectors_ids) > 0

            arl_citysectors_ids = await api.fetch_airline_citysectors(scen_id, arl.id)
            assert len(arl_citysectors_ids) > 0

            arl_routes_ids = await api.fetch_airline_routes(scen_id, arl.id)
            assert len(arl_routes_ids) > 0

            arl_cityroutes_ids = await api.fetch_airline_cityroutes(scen_id, arl.id)
            assert len(arl_cityroutes_ids) > 0

        arc_items = await api.fetch_aircrafts(scen_id)
        assert len(arc_items) > 0

        arp_items = await api.fetch_airports(scen_id)
        assert len(arp_items) > 0

        cnt_items = await api.fetch_countries(scen_id)
        assert len(cnt_items) > 0

        cty_items = await api.fetch_cities(scen_id)
        assert len(cty_items) > 0

        pfl_items = await api.fetch_parametric_filters(scen_id)
        assert len(pfl_items) > 0

        pmd_items = await api.fetch_parametric_models(scen_id)
        assert len(pmd_items) > 0

        # ---- Assert all derived items are initialized
        csc_items = await api.fetch_citysectors(scen_id)
        assert len(csc_items) > 0

        crt_items = await api.fetch_cityroutes(scen_id)
        assert len(crt_items) > 0

        sct_items = await api.fetch_sectors(scen_id)
        assert len(sct_items) > 0

        rte_items = await api.fetch_routes(scen_id)
        assert len(rte_items) > 0

        # ---- Assert all unbounded items are initialized (by citysector)
        for csc in csc_items:

            sch_ids = await api.fetch_schedules_ids(scen_id, citysector_id=csc.id)
            assert len(sch_ids) > 0

            sch_items = await api.fetch_schedules(
                scen_id, sch_ids, citysector_id=csc.id
            )
            assert len(sch_items) > 0

            flt_ids = await api.fetch_flights_ids(scen_id, citysector_id=csc.id)
            assert len(flt_ids) > 0

            flt_items = await api.fetch_flights(scen_id, flt_ids, citysector_id=csc.id)
            assert len(flt_items) > 0

        # ---- Assert all unbounded items are initialized (by sector)
        for sct in sct_items:

            sch_ids = await api.fetch_schedules_ids(scen_id, sector_id=sct.id)
            assert len(sch_ids) > 0

            sch_items = await api.fetch_schedules(scen_id, sch_ids, sector_id=sct.id)
            assert len(sch_items) > 0

            flt_ids = await api.fetch_flights_ids(scen_id, sector_id=sct.id)
            assert len(flt_ids) > 0

            flt_items = await api.fetch_flights(scen_id, flt_ids, sector_id=sct.id)
            assert len(flt_items) > 0



from rmlab._api.base import APIBaseInternal
from rmlab._data.enums import (
    DataRemoveKind,
)


class APIRemoveInternal(APIBaseInternal):
    """Interface to remove data from server."""

    async def _remove_data(self, scen_id: int, kind: DataRemoveKind) -> None:
        """Remove some or all the data of a scenario.

        Args:
            scen_id (int): Scenario ID.
            kind (DataRemoveKind): Type of removal.
        """

        await self._submit_call(
            resource=self._api_endpoints.data_reset,
            verb="post",
            data={"scen_id": str(scen_id), "kind": kind.value},
        )



import os, logging
from typing import Any, List, Mapping, Optional, Union

from rmlab_errors import LogicError
from rmlab_http_client import (
    AsyncClient,
    HTTPClientBasic,
    HTTPClientJWTRenewable,
    HTTPClientJWT,
)
from rmlab._api.endpoints import (
    ApiEndpoints,
    ApiEndpointsLimits,
    BaseURL,
    ResourceUserLogin,
    ResourceGetEndpoints,
)

from rmlab._version import __version__

_Logger = logging.getLogger(__name__)

_ServicesType = List[str]
_ScenariosType = Mapping[int, str]

_UndefinedCredentials: Mapping[str, Union[str, _ServicesType, _ScenariosType]] = {
    "access_token": None,
    "refresh_token": None,
    "services": None,
    "scenarios": None,
    "username": None,
    "workgroup": None,
}


class APIBaseInternal:
    """Asynchronous context manager providing internal initialization, login and API server calls to derived classes."""

    def __init__(
        self,
        workgroup: Optional[str] = None,
        username: Optional[str] = None,
        password: Optional[str] = None,
    ) -> None:
        """Initializes APIBaseInternal instance.

        Args:
            workgroup: (str, optional): Workgroup name. Defaults to None.
                If none, expects `RMLAB_WORKGROUP` environment variable definition.
            user (str, optional): User name. Defaults to None.
                If none, expects `RMLAB_USERNAME` environment variable definition.
            password (str, optional): Password. Defaults to None.
                If none, expects `RMLAB_PASSWORD` environment variable definition..

        Raises:
            AuthenticationError: If user or password are not defined.
        """

        if any([arg is None for arg in [workgroup, username, password]]):

            try:
                workgroup = os.environ["RMLAB_WORKGROUP"]
                username = os.environ["RMLAB_USERNAME"]
                password = os.environ["RMLAB_PASSWORD"]

            except KeyError:

                raise ValueError(
                    f"`RMLAB_WORKGROUP` / `RMLAB_USERNAME` / `RMLAB_PASSWORD` are undefined"
                )

        # Prepare login data to be used at __aenter__
        self._login_data = {
            "workgroup": workgroup,
            "username": username,
            "password": password,
            "version": __version__,
        }

        self._credentials = _UndefinedCredentials

        self._api_endpoints = ApiEndpoints()
        self._url_api = os.environ.get("RMLAB_URL_API") or BaseURL
        self._url_auth = os.environ.get("RMLAB_URL_AUTH") or BaseURL

    @property
    def _auth_data(self):
        return ":".join(self._login_data.values())

    @property
    def _workgroup(self):
        return self._credentials["workgroup"]

    @property
    def scenarios(self):
        return [int(sc_id) for sc_id in self._credentials["scenarios"].keys()]

    async def __aenter__(self):
        """Context manager asynchronous enter. Log in and credentials initialization.

        Returns:
            APIBaseInternal: This instance.
        """
        """"""

        login_client = AsyncClient(
            HTTPClientBasic,
            timeout_seconds=120,
            op_address=self._url_auth,
            op_basic=self._auth_data,
            address=self._url_auth,
            auth_data=self._auth_data,
        )

        _Logger.debug("Logging-in...")

        self._credentials = await login_client.submit_request(
            resource=ResourceUserLogin, verb="post", return_type="json"
        )

        async with HTTPClientJWT(
            address=self._url_api, jwt=self._credentials["access_token"]
        ) as client:

            resp = await client.submit_request(
                resource="/" + self._credentials["workgroup"] + ResourceGetEndpoints,
                return_type="json",
            )

            endpoints_json = resp["endpoints"]
            endpoints_limits_json = resp["endpoints_limits"]

        self._api_endpoints = ApiEndpoints(**endpoints_json)
        self._api_endpoints_limits = ApiEndpointsLimits(**endpoints_limits_json)

        _Logger.debug("Logged-in")

        return self

    async def __aexit__(self, exc_ty, exc_val, exc_tb):
        """Context manager asynchronous exit. Re-raise any error"""

        try:

            async with HTTPClientBasic(
                address=self._url_auth, auth_data=self._auth_data
            ) as logout_client:

                await logout_client.submit_request(
                    resource="/auth/user/logout", verb="post"
                )

            self._credentials = _UndefinedCredentials

            self._api_endpoints = ApiEndpoints()

        except Exception:
            # Do not propagate exceptions due to log-out
            pass

        # Do not handle other exceptions here, return None to re-raise

    async def _submit_call(self, is_async: Optional[bool] = False, **kwargs) -> Any:
        """Performs API call to server, forwarding keyword arguments.

        Args:
            is_async (Optional[bool], optional): Whether the resource is asynchronousor not. Defaults to False.

        Raises:
            LogicError: If credentials are not defined

        Returns:
            Any: The response of the API call.
        """

        if not is_async:

            # Synchronous API call

            if (
                self._credentials["access_token"] is None
                or self._credentials["refresh_token"] is None
            ):
                raise LogicError(f"Cannot submit API call, credentials undefined")

            async with HTTPClientJWTRenewable(
                address=self._url_api,
                access_jwt=self._credentials["access_token"],
                auth_address=self._url_auth,
                refresh_jwt=self._credentials["refresh_token"],
            ) as client:

                return await client.submit_request(**kwargs)

        else:

            # Asynchronous API call

            async with AsyncClient(
                HTTPClientJWTRenewable,
                op_address=self._url_api,
                op_jwt=self._credentials["access_token"],
                address=self._url_api,
                access_jwt=self._credentials["access_token"],
                auth_address=self._url_auth,
                refresh_jwt=self._credentials["refresh_token"],
            ) as async_client:

                return await async_client.submit_request(**kwargs)



import os, pathlib
from rmlab.api.fetch.core import APIFetchCore

from rmlab.api.remove import APIRemove
from rmlab.api.operations.simulation import APISimulation
from rmlab.api.upload.core import APIUploadCore
from rmlab.api.upload.parametric.customers import APIUploadCustomersModels
from rmlab.api.upload.parametric.filters import APIUploadParametric
from rmlab.api.upload.parametric.pricing import APIUploadPricingModels
from rmlab.api.fetch.flight_data import APIFetchFlightData

_DataSamplesPath = (
    str(pathlib.Path(os.path.realpath(__file__)).parent.parent) + "/data_samples/"
)


class APISimulationUpload(
    APISimulation,
    APIFetchCore,
    APIFetchFlightData,
    APIUploadCore,
    APIUploadParametric,
    APIUploadCustomersModels,
    APIUploadPricingModels,
    APIRemove,
):
    pass


async def test_analysis():

    async with APISimulationUpload() as api:

        scen_id = api.scenarios[0]

        await api.remove_data_full(scen_id=scen_id)

        # Upload customers models to server
        await api.upload_batch_customers_models(
            scen_id,
            request_models_fns=[
                _DataSamplesPath + "/basic/customers_request.poisson_bl1.json"
            ],
            choice_models_fns=[
                _DataSamplesPath + "/basic/customers_choice.mnl_bl1.json"
            ],
        )

        # Upload pricing models to server
        await api.upload_batch_pricing_models(
            scen_id,
            range_models_fns=[_DataSamplesPath + "/basic/pricing_range.sample.json"],
            behavior_models_fns=[
                _DataSamplesPath + "/basic/pricing_behavior.sample.json"
            ],
            optimizer_models_fns=[
                _DataSamplesPath + "/basic/pricing_optimizer.sample.json"
            ],
        )

        await api.upload_parametric_filters(
            scen_id, data_fn=_DataSamplesPath + "/basic/table.pfilter.json"
        )

        # Upload core items to server
        await api.upload_batch_core(
            scen_id,
            aircraft_items_fn=_DataSamplesPath + "/basic/table.aircraft.csv",
            airline_items_fn=_DataSamplesPath + "/basic/table.airline.csv",
            airport_items_fn=_DataSamplesPath + "/basic/table.airport.csv",
            city_items_fn=_DataSamplesPath + "/basic/table.city.csv",
            country_items_fn=_DataSamplesPath + "/basic/table.country.csv",
            schedule_items_fn=_DataSamplesPath + "/basic/table.schedule.csv",
        )

        await api.trigger_simulation(scen_id)

        csc_items = await api.fetch_citysectors(scen_id)
        assert len(csc_items) > 0

        for csc in csc_items:

            flt_ids = await api.fetch_flights_ids(scen_id, citysector_id=csc.id)
            assert len(flt_ids) > 0

            actual_books = await api.fetch_flights_data_historic(
                scen_id, flt_ids, citysector_id=csc.id
            )
            assert all([not flt_data.is_empty() for flt_data in actual_books])

            threshold_setts = await api.fetch_flights_data_pricing_thresholds(
                scen_id, flt_ids, citysector_id=csc.id
            )
            assert all([not flt_data.is_empty() for flt_data in threshold_setts])

            pps_setts = await api.fetch_flights_data_pricing_per_seat(
                scen_id, flt_ids, citysector_id=csc.id
            )
            assert all([not flt_data.is_empty() for flt_data in pps_setts])

            events = await api.fetch_flights_data_events(
                scen_id, flt_ids, citysector_id=csc.id
            )
            assert all([not flt_data.is_empty() for flt_data in events])



"""Interface to trigger optimization passes."""

from datetime import datetime
from typing import Optional
from rmlab._api.base import APIBaseInternal
from rmlab._data.types import DateTimeMinFormat


class APIOptimization(APIBaseInternal):
    """Exposes functions to run and schedule optimization runs on a set of flights."""

    async def trigger_optimization_pass(
        self,
        scen_id: int,
        airline_id: str,
        *,
        citysector_id: Optional[str] = None,
        sector_id: Optional[str] = None,
    ):
        """Triggers an optimization pass on all flights of an airline belonging to a citysector or sector.

        Args:
            scen_id (int): Scenario ID
            airline_id (str): Airline ID
            citysector_id (Optional[str], optional): Citysector ID. Defaults to None.
            sector_id (Optional[str], optional): Sector ID. Defaults to None.

        Raises:
            ValueError: If none of `citysector_id`, `sector_id` is defined
        """

        if citysector_id is None and sector_id is None:
            raise ValueError(
                f"At least one of `citysector_id`, `sector_id` must be defined"
            )

        await self._submit_call(
            is_async=True,
            resource=self._api_endpoints.operation_trigger,
            verb="post",
            data={
                "scen_id": scen_id,
                "operation": "rms-fine",
                "airline_id": airline_id,
                "citysector_id": str(citysector_id),
                "sector_id": str(sector_id),
            },
        )

    async def schedule_optimization_pass(
        self,
        scen_id: int,
        airline_id: str,
        date_time: datetime,
        *,
        citysector_id: Optional[str] = None,
        sector_id: Optional[str] = None,
    ):
        """Schedules an optimization pass on all flights of an airline belonging to a citysector or sector to be run at specific date and time.

        Args:
            scen_id (int): Scenario ID
            airline_id (str): Airline ID
            date_time (datetime): Date and time at which the optimization pass is triggered.
            citysector_id (Optional[str], optional): Citysector ID. Defaults to None.
            sector_id (Optional[str], optional): Sector ID. Defaults to None.

        Raises:
            ValueError: If none of `citysector_id`, `sector_id` is defined
        """

        if citysector_id is None and sector_id is None:
            raise ValueError(
                f"At least one of `citysector_id`, `sector_id` must be defined"
            )

        await self._submit_call(
            is_async=True,
            resource=self._api_endpoints.operation_trigger,
            verb="post",
            data={
                "scen_id": scen_id,
                "operation": "rms-schedule",
                "datetime": datetime.strftime(date_time, DateTimeMinFormat),
                "airline_id": airline_id,
                "citysector_id": str(citysector_id),
                "sector_id": str(sector_id),
            },
        )



import os, pytest
from dataclasses import dataclass
from urllib.request import Request, urlopen
from urllib.error import HTTPError
from rmlab_http_client import AsyncClient, HTTPClientBasic
from rmlab._api.base import APIBaseInternal
from rmlab._api.endpoints import BaseURL
from rmlab_errors import AuthenticationError
from rmlab._version import __version__

import os


@dataclass
class TestLoginData:
    workgroup: str
    username: str
    password: str


@dataclass
class TestBaseURLs:
    api: str
    auth: str


@pytest.fixture
def base_urls():

    return TestBaseURLs(
        api=os.environ.get("RMLAB_URL_API") or BaseURL,
        auth=os.environ.get("RMLAB_URL_AUTH") or BaseURL,
    )


@pytest.fixture
def login_data():

    return TestLoginData(
        workgroup=os.environ["RMLAB_WORKGROUP"],
        username=os.environ["RMLAB_USERNAME"],
        password=os.environ["RMLAB_PASSWORD"],
    )


def test_web_is_up(base_urls):

    headers = {"User-Agent": "Mozilla/5.0"}

    if BaseURL in base_urls.api:
        # Expect correct html response
        assert urlopen(Request(base_urls.api, headers=headers)).getcode() == 200
        assert urlopen(Request(base_urls.auth, headers=headers)).getcode() == 200
    else:
        # Testing/Dev API only. Expect not found (404) response since '/' resource is not defined
        with pytest.raises(HTTPError):
            urlopen(Request(base_urls.api, headers=headers))
            urlopen(Request(base_urls.auth, headers=headers))


async def test_log_in_out(login_data, base_urls):

    auth_data = ":".join(list(login_data.__dict__.values()) + [__version__])

    login_client = AsyncClient(
        HTTPClientBasic,
        timeout_seconds=180,
        op_address=base_urls.auth,
        op_basic=auth_data,
        address=base_urls.auth,
        auth_data=auth_data,
    )

    login_resp = await login_client.submit_request(
        resource="/auth/user/login", verb="post", return_type="json"
    )

    assert "access_token" in login_resp
    assert "refresh_token" in login_resp
    assert "services" in login_resp
    assert "scenarios" in login_resp
    assert "username" in login_resp
    assert "workgroup" in login_resp

    async with HTTPClientBasic(
        address=base_urls.auth, auth_data=auth_data
    ) as logout_client:

        logout_resp = await logout_client.submit_request(
            resource="/auth/user/logout", verb="post"
        )

        assert logout_resp is None


async def test_session_args(login_data):

    async with APIBaseInternal(**login_data.__dict__):
        pass


async def test_session_env_vars(login_data):

    async with APIBaseInternal():
        pass


async def test_session_fail(login_data):

    with pytest.raises(AuthenticationError):
        async with APIBaseInternal(
            workgroup="wrong-workgroup", username="wrong-user", password="wrong-pwd"
        ):
            pass
